package com.ruoyi.web.controller.buyerapi;

import java.util.List;
import cn.hutool.core.util.IdUtil;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.buyerapi.domain.BuyerDeptMaterial;
import com.ruoyi.buyerapi.service.IBuyerDeptMaterialService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 项目材料领用记录Controller
 * 
 * @author ruoyi
 * @date 2022-09-22
 */
@Api(tags = "项目材料领用记录")
@RestController
@RequestMapping("/buyer/material")
public class BuyerDeptMaterialController extends BaseController
{
    @Autowired
    private IBuyerDeptMaterialService buyerDeptMaterialService;

    /**
     * 查询项目材料领用记录列表
     */
    @ApiOperation(value = "查询项目材料领用记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "collectDate", value = "领用日期", required = true, paramType = "query"),
            @ApiImplicitParam(name = "projectName", value = "项目名称", required = true, paramType = "query"),
            @ApiImplicitParam(name = "name", value = "物料名称", required = true, paramType = "query"),
            @ApiImplicitParam(name = "approvaler", value = "审批人", required = true, paramType = "query"),
            @ApiImplicitParam(name = "collecter", value = "领料人", required = true, paramType = "query")
    })
    @PreAuthorize("@ss.hasPermi('buyer:material:list')")
    @GetMapping("/list")
    public TableDataInfo list(BuyerDeptMaterial buyerDeptMaterial)
    {
        startPage();
        List<BuyerDeptMaterial> list = buyerDeptMaterialService.selectBuyerDeptMaterialList(buyerDeptMaterial);
        return getDataTable(list);
    }

    /**
     * 统计项目材料领用记录列表
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "projectName", value = "项目名称", required = true, paramType = "query")
    })
    @ApiOperation(value = "统计项目材料领用记录列表")
    @GetMapping("/statist")
    public TableDataInfo statist(BuyerDeptMaterial buyerDeptMaterial)
    {
        startPage();
        List<BuyerDeptMaterial> list = buyerDeptMaterialService.selectStatistBuyerDeptMaterialList(buyerDeptMaterial);
        return getDataTable(list);
    }

    /**
     * 导出项目材料领用记录列表
     */
    @ApiOperation(value = "导出项目材料领用记录列表")
    @PreAuthorize("@ss.hasPermi('buyer:material:export')")
    @Log(title = "项目材料领用记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BuyerDeptMaterial buyerDeptMaterial)
    {
        List<BuyerDeptMaterial> list = buyerDeptMaterialService.selectBuyerDeptMaterialList(buyerDeptMaterial);
        ExcelUtil<BuyerDeptMaterial> util = new ExcelUtil<BuyerDeptMaterial>(BuyerDeptMaterial.class);
        util.exportExcel(response, list, "项目材料领用记录数据");
    }

    /**
     * 获取项目材料领用记录详细信息
     */
    @ApiOperation(value = "获取项目材料领用记录详细信息")
    @PreAuthorize("@ss.hasPermi('buyer:material:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(buyerDeptMaterialService.selectBuyerDeptMaterialById(id));
    }

    /**
     * 新增项目材料领用记录
     */
    @ApiOperation(value = "新增项目材料领用记录")
    @PreAuthorize("@ss.hasPermi('buyer:material:add')")
    @Log(title = "项目材料领用记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BuyerDeptMaterial buyerDeptMaterial)
    {
        buyerDeptMaterial.setId(IdUtil.simpleUUID());
        buyerDeptMaterial.setCreateBy(getUsername());
        return toAjax(buyerDeptMaterialService.insertBuyerDeptMaterial(buyerDeptMaterial));
    }

    /**
     * 修改项目材料领用记录
     */
    @ApiOperation(value = "修改项目材料领用记录")
    @PreAuthorize("@ss.hasPermi('buyer:material:edit')")
    @Log(title = "项目材料领用记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BuyerDeptMaterial buyerDeptMaterial)
    {
        buyerDeptMaterial.setUpdateBy(getUsername());
        return toAjax(buyerDeptMaterialService.updateBuyerDeptMaterial(buyerDeptMaterial));
    }

    /**
     * 删除项目材料领用记录
     */
    @ApiOperation(value = "删除项目材料领用记录")
    @PreAuthorize("@ss.hasPermi('buyer:material:remove')")
    @Log(title = "项目材料领用记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(buyerDeptMaterialService.deleteBuyerDeptMaterialByIds(ids));
    }
}
