package com.ruoyi.web.controller.buyerapi;

import java.util.List;
import cn.hutool.core.util.IdUtil;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.buyerapi.domain.BuyerOtherCost;
import com.ruoyi.buyerapi.service.IBuyerOtherCostService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 项目其他费用记录Controller
 * 
 * @author ruoyi
 * @date 2022-09-22
 */
@Api(tags = "项目其他费用记录")
@RestController
@RequestMapping("/buyer/othercost")
public class BuyerOtherCostController extends BaseController
{
    @Autowired
    private IBuyerOtherCostService buyerOtherCostService;

    /**
     * 查询项目其他费用记录列表
     */
    @ApiOperation(value = "查询项目其他费用记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "collectDate", value = "领用日期", required = true, paramType = "query"),
            @ApiImplicitParam(name = "projectName", value = "项目名称", required = true, paramType = "query"),
            @ApiImplicitParam(name = "info", value = "摘要", required = true, paramType = "query"),
            @ApiImplicitParam(name = "type", value = "种类", required = true, paramType = "query"),
            @ApiImplicitParam(name = "createBy", value = "操作人", required = true, paramType = "query")
    })
    @PreAuthorize("@ss.hasPermi('buyer:othercost:list')")
    @GetMapping("/list")
    public TableDataInfo list(BuyerOtherCost buyerOtherCost)
    {
        startPage();
        List<BuyerOtherCost> list = buyerOtherCostService.selectBuyerOtherCostList(buyerOtherCost);
        return getDataTable(list);
    }

    /**
     * 统计项目其他费用记录列表
     */
    @ApiOperation(value = "统计项目其他费用记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "projectName", value = "项目名称", required = true, paramType = "query")
    })
    @GetMapping("/statist")
    public TableDataInfo statist(BuyerOtherCost buyerOtherCost)
    {
        startPage();
        List<BuyerOtherCost> list = buyerOtherCostService.selectStatistBuyerOtherCostList(buyerOtherCost);
        return getDataTable(list);
    }

    /**
     * 导出项目其他费用记录列表
     */
    @ApiOperation(value = "导出项目其他费用记录列表")
    @PreAuthorize("@ss.hasPermi('buyer:othercost:export')")
    @Log(title = "项目其他费用记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BuyerOtherCost buyerOtherCost)
    {
        List<BuyerOtherCost> list = buyerOtherCostService.selectBuyerOtherCostList(buyerOtherCost);
        ExcelUtil<BuyerOtherCost> util = new ExcelUtil<BuyerOtherCost>(BuyerOtherCost.class);
        util.exportExcel(response, list, "项目其他费用记录数据");
    }

    /**
     * 获取项目其他费用记录详细信息
     */
    @ApiOperation(value = "获取项目其他费用记录详细信息")
    @PreAuthorize("@ss.hasPermi('buyer:othercost:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(buyerOtherCostService.selectBuyerOtherCostById(id));
    }

    /**
     * 新增项目其他费用记录
     */
    @ApiOperation(value = "新增项目其他费用记录")
    @PreAuthorize("@ss.hasPermi('buyer:othercost:add')")
    @Log(title = "项目其他费用记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BuyerOtherCost buyerOtherCost)
    {
        buyerOtherCost.setId(IdUtil.simpleUUID());
        buyerOtherCost.setCreateBy(getUsername());
        return toAjax(buyerOtherCostService.insertBuyerOtherCost(buyerOtherCost));
    }

    /**
     * 修改项目其他费用记录
     */
    @ApiOperation(value = "修改项目其他费用记录")
    @PreAuthorize("@ss.hasPermi('buyer:othercost:edit')")
    @Log(title = "项目其他费用记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BuyerOtherCost buyerOtherCost)
    {
        buyerOtherCost.setUpdateBy(getUsername());
        return toAjax(buyerOtherCostService.updateBuyerOtherCost(buyerOtherCost));
    }

    /**
     * 删除项目其他费用记录
     */
    @ApiOperation(value = "删除项目其他费用记录")
    @PreAuthorize("@ss.hasPermi('buyer:othercost:remove')")
    @Log(title = "项目其他费用记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(buyerOtherCostService.deleteBuyerOtherCostByIds(ids));
    }
}
