package com.ruoyi.web.controller.buyerapi;

import java.util.List;
import cn.hutool.core.util.IdUtil;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.buyerapi.domain.BuyerOutsourcing;
import com.ruoyi.buyerapi.service.IBuyerOutsourcingService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 委外研发记录Controller
 * 
 * @author ruoyi
 * @date 2022-09-22
 */
@Api(tags = "委外研发记录")
@RestController
@RequestMapping("/buyer/outsourcing")
public class BuyerOutsourcingController extends BaseController
{
    @Autowired
    private IBuyerOutsourcingService buyerOutsourcingService;

    /**
     * 查询委外研发记录列表
     */
    @ApiOperation(value = "查询委外研发记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "collectDate", value = "领用日期", required = true, paramType = "query"),
            @ApiImplicitParam(name = "projectName", value = "项目名称", required = true, paramType = "query"),
            @ApiImplicitParam(name = "info", value = "摘要", required = true, paramType = "query"),
            @ApiImplicitParam(name = "type", value = "种类", required = true, paramType = "query"),
            @ApiImplicitParam(name = "createBy", value = "操作人", required = true, paramType = "query")
    })
    @PreAuthorize("@ss.hasPermi('buyer:outsourcing:list')")
    @GetMapping("/list")
    public TableDataInfo list(BuyerOutsourcing buyerOutsourcing)
    {
        startPage();
        List<BuyerOutsourcing> list = buyerOutsourcingService.selectBuyerOutsourcingList(buyerOutsourcing);
        return getDataTable(list);
    }

    /**
     * 统计委外研发记录列表
     */
    @ApiOperation(value = "统计委外研发记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "projectName", value = "项目名称", required = true, paramType = "query")
    })
    @GetMapping("/statist")
    public TableDataInfo statist(BuyerOutsourcing buyerOutsourcing)
    {
        startPage();
        List<BuyerOutsourcing> list = buyerOutsourcingService.selectStatistBuyerOutsourcingList(buyerOutsourcing);
        return getDataTable(list);
    }

    /**
     * 导出委外研发记录列表
     */
    @ApiOperation(value = "导出委外研发记录列表")
    @PreAuthorize("@ss.hasPermi('buyer:outsourcing:export')")
    @Log(title = "委外研发记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BuyerOutsourcing buyerOutsourcing)
    {
        List<BuyerOutsourcing> list = buyerOutsourcingService.selectBuyerOutsourcingList(buyerOutsourcing);
        ExcelUtil<BuyerOutsourcing> util = new ExcelUtil<BuyerOutsourcing>(BuyerOutsourcing.class);
        util.exportExcel(response, list, "委外研发记录数据");
    }

    /**
     * 获取委外研发记录详细信息
     */
    @ApiOperation(value = "获取委外研发记录详细信息")
    @PreAuthorize("@ss.hasPermi('buyer:outsourcing:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(buyerOutsourcingService.selectBuyerOutsourcingById(id));
    }

    /**
     * 新增委外研发记录
     */
    @ApiOperation(value = "新增委外研发记录")
    @PreAuthorize("@ss.hasPermi('buyer:outsourcing:add')")
    @Log(title = "委外研发记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BuyerOutsourcing buyerOutsourcing)
    {
        buyerOutsourcing.setId(IdUtil.simpleUUID());
        buyerOutsourcing.setCreateBy(getUsername());
        return toAjax(buyerOutsourcingService.insertBuyerOutsourcing(buyerOutsourcing));
    }

    /**
     * 修改委外研发记录
     */
    @ApiOperation(value = "修改委外研发记录")
    @PreAuthorize("@ss.hasPermi('buyer:outsourcing:edit')")
    @Log(title = "委外研发记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BuyerOutsourcing buyerOutsourcing)
    {
        buyerOutsourcing.setUpdateBy(getUsername());
        return toAjax(buyerOutsourcingService.updateBuyerOutsourcing(buyerOutsourcing));
    }

    /**
     * 删除委外研发记录
     */
    @ApiOperation(value = "删除委外研发记录")
    @PreAuthorize("@ss.hasPermi('buyer:outsourcing:remove')")
    @Log(title = "委外研发记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(buyerOutsourcingService.deleteBuyerOutsourcingByIds(ids));
    }
}
