package com.ruoyi.web.controller.buyerapi;

import java.util.List;
import cn.hutool.core.util.IdUtil;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.buyerapi.domain.BuyerProjectAccept;
import com.ruoyi.buyerapi.service.IBuyerProjectAcceptService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 立项管理验收资料Controller
 * 
 * @author ruoyi
 * @date 2022-09-22
 */
@Api(tags = "立项管理验收资料")
@RestController
@RequestMapping("/buyer/acceptfile")
public class BuyerProjectAcceptController extends BaseController
{
    @Autowired
    private IBuyerProjectAcceptService buyerProjectAcceptService;

    /**
     * 查询立项管理验收资料列表
     */
    @ApiOperation(value = "查询立项管理验收资料列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "projectName", value = "项目名称", required = true, paramType = "query"),
            @ApiImplicitParam(name = "name", value = "验收名称", required = true, paramType = "query"),
            @ApiImplicitParam(name = "date", value = "验收日期", required = true, paramType = "query"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = true, paramType = "query")
    })
    @PreAuthorize("@ss.hasPermi('buyer:acceptfile:list')")
    @GetMapping("/list")
    public TableDataInfo list(BuyerProjectAccept buyerProjectAccept)
    {
        startPage();
        List<BuyerProjectAccept> list = buyerProjectAcceptService.selectBuyerProjectAcceptList(buyerProjectAccept);
        return getDataTable(list);
    }

    /**
     * 导出立项管理验收资料列表
     */
    @ApiOperation(value = "导出立项管理验收资料列表")
    @PreAuthorize("@ss.hasPermi('buyer:acceptfile:export')")
    @Log(title = "立项管理验收资料", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BuyerProjectAccept buyerProjectAccept)
    {
        List<BuyerProjectAccept> list = buyerProjectAcceptService.selectBuyerProjectAcceptList(buyerProjectAccept);
        ExcelUtil<BuyerProjectAccept> util = new ExcelUtil<BuyerProjectAccept>(BuyerProjectAccept.class);
        util.exportExcel(response, list, "立项管理验收资料数据");
    }

    /**
     * 获取立项管理验收资料详细信息
     */
    @ApiOperation(value = "获取立项管理验收资料详细信息")
    @PreAuthorize("@ss.hasPermi('buyer:acceptfile:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(buyerProjectAcceptService.selectBuyerProjectAcceptById(id));
    }

    /**
     * 新增立项管理验收资料
     */
    @ApiOperation(value = "新增立项管理验收资料")
    @PreAuthorize("@ss.hasPermi('buyer:acceptfile:add')")
    @Log(title = "立项管理验收资料", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BuyerProjectAccept buyerProjectAccept)
    {
        buyerProjectAccept.setId(IdUtil.simpleUUID());
        buyerProjectAccept.setCreateBy(getUsername());
        return toAjax(buyerProjectAcceptService.insertBuyerProjectAccept(buyerProjectAccept));
    }

    /**
     * 修改立项管理验收资料
     */
    @ApiOperation(value = "修改立项管理验收资料")
    @PreAuthorize("@ss.hasPermi('buyer:acceptfile:edit')")
    @Log(title = "立项管理验收资料", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BuyerProjectAccept buyerProjectAccept)
    {
        buyerProjectAccept.setUpdateBy(getUsername());
        return toAjax(buyerProjectAcceptService.updateBuyerProjectAccept(buyerProjectAccept));
    }

    /**
     * 删除立项管理验收资料
     */
    @ApiOperation(value = "删除立项管理验收资料")
    @PreAuthorize("@ss.hasPermi('buyer:acceptfile:remove')")
    @Log(title = "立项管理验收资料", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(buyerProjectAcceptService.deleteBuyerProjectAcceptByIds(ids));
    }
}
